@@ -388,7 +388,7 @@ SparseMatrix<T>::SparseMatrix(std::string fname,unsigned char vtype,char csep) :
     this->nr=0;
     while (!this->ifile.eof())
     {
-        this->ifile >> line;
+        getline(this->ifile,line);
         if (!this->ifile.eof())
             this->nr++;
     }
@@ -409,7 +409,7 @@ SparseMatrix<T>::SparseMatrix(std::string fname,unsigned char vtype,char csep) :
     this->ifile.close();
     this->ifile.open(fname.c_str());
     size_t p=0;
-    this->ifile >> line;
+    getline(this->ifile,line);
     // No need to process first line here, it was done at the parent's class constructor
     
     T *data_with_zeros = new T [this->nc];
@@ -417,9 +417,16 @@ SparseMatrix<T>::SparseMatrix(std::string fname,unsigned char vtype,char csep) :
     std::vector<indextype> datacolsofrow;
     std::vector<T>dataofrow;
     
+    if (DEB & DEBJM)
+        Rcpp::Rcout << "Reading line... ";
     while (!this->ifile.eof())
     {
-        this->ifile >> line;
+        if ( (DEB & DEBJM) && !(p%1000))
+        {
+            Rcpp::Rcout << p << " ";
+            Rcpp::Rcout.flush();
+        }
+        getline(this->ifile,line);
         if (!this->ifile.eof())
         {
           if (!this->ProcessDataLineCsv(line,csep,data_with_zeros))
@@ -446,7 +453,7 @@ SparseMatrix<T>::SparseMatrix(std::string fname,unsigned char vtype,char csep) :
      
     if (DEB & DEBJM)
     {
-        Rcpp::Rcout << "Read " << p << " data lines of file " << fname;
+        Rcpp::Rcout << "\nRead " << p << " data lines of file " << fname;
         if (p==this->nr)
             Rcpp::Rcout << ", as expected.\n";
         else
@@ -466,7 +473,7 @@ TEMPLATES_CONST(SparseMatrix,SINGLE_ARG(std::string fname,unsigned char vtype,ch
 template <typename T>
 T SparseMatrix<T>::Get(indextype r,indextype c) const
 { 
-#ifdef WITH_CHECKS_MATRIXSP
+#ifdef WITH_CHECKS_MATRIX
     if ((r>=this->nr) || (c>=this->nc))
     {
         std::ostringstream errst;
@@ -512,7 +519,7 @@ TEMPLATES_FUNCRCONST(SparseMatrix,Get,SINGLE_ARG(indextype r,indextype c))
 template <typename T>
 void SparseMatrix<T>::Set(indextype r,indextype c,T v)
 {
-#ifdef WITH_CHECKS_MATRIXSP
+#ifdef WITH_CHECKS_MATRIX
     if ((r>=this->nr) || (c>=this->nc))
     {
     	std::ostringstream errst;
@@ -574,7 +581,7 @@ TEMPLATES_SETFUNC(void,SparseMatrix,Set,SINGLE_ARG(indextype r,indextype c),v)
 template <typename T>
 void SparseMatrix<T>::SetRow(indextype r,std::vector<indextype> vc,std::vector<T> v)
 {
-#ifdef WITH_CHECKS_MATRIXSP
+#ifdef WITH_CHECKS_MATRIX
     if ((r>=this->nr) || (vc.size()>=this->nc))
     {
         std::ostringstream errst;
@@ -595,7 +602,7 @@ TEMPLATES_SETFUNCVEC(void,SparseMatrix,SetRow,SINGLE_ARG(indextype r,std::vector
 template <typename T>
 void SparseMatrix<T>::GetRow(indextype r,T *v)
 {
-#ifdef WITH_CHECKS_MATRIXSP
+#ifdef WITH_CHECKS_MATRIX
     if (r>=this->nr)
     {
     	std::ostringstream errst;
@@ -616,7 +623,7 @@ TEMPLATES_SETFUNC(void,SparseMatrix,GetRow,indextype r,*v)
 template <typename T>
 void SparseMatrix<T>::GetSparseRow(indextype r,unsigned char *m,unsigned char s,T *v)
 {
-#ifdef WITH_CHECKS_MATRIXSP
+#ifdef WITH_CHECKS_MATRIX
     if (r>=this->nr)
     {
         std::ostringstream errst;
@@ -640,7 +647,7 @@ TEMPLATES_SETFUNC(void,SparseMatrix,GetSparseRow,SINGLE_ARG(indextype r,unsigned
 template <typename T>
 void SparseMatrix<T>::GetMarksOfSparseRow(indextype r,unsigned char *m,unsigned char s)
 {
-#ifdef WITH_CHECKS_MATRIXSP
+#ifdef WITH_CHECKS_MATRIX
     if (r>=this->nr)
     {
     	std::ostringstream errst;
@@ -780,27 +787,38 @@ TEMPLATES_FUNC(void,SparseMatrix,WriteBin,std::string fname)
 template <typename T>
 void SparseMatrix<T>::WriteCsv(std::string fname,char csep,bool withquotes)
 {
+    // Remember: this writes the header, even if there were no column names (then, the header will be "","C1","C2",...)
     ((JMatrix<T> *)this)->WriteCsv(fname,csep,withquotes);
-    
-    bool with_headers=false;
-    size_t nch=this->colnames.size();
-    size_t nrh=this->rownames.size();
-    
-    if (nch>0 && nrh>0)
+
+    // Header has been written (unless the matrix had no columns...)
+    // But if it would have columns, but no rows, there is nothing after the header. The .csv would have a single line (the header)
+    if ((this->nc==0) || (this->nr==0))
     {
-     if (nch!=this->nc || nrh!=this->nr)
-      Rcpp::warning("Different size of headers and matrix, either in rows or in columns. Headers will not be written in the .csv file.\n");
-     with_headers=true;
+     this->ofile.close();
+     return;
     }
-   
+
+    int p = std::numeric_limits<T>::max_digits10;
+
+    // We have rows to write; otherwise we would have returned four lines ago...
+    indextype rns=this->rownames.size();
+    
     for (indextype r=0;r<this->nr;r++)
     {
-        if (with_headers)
+        if (rns>0)
             this->ofile << FixQuotes(this->rownames[r],withquotes) << csep;
-           
+        else
+        {
+         if (withquotes)
+            this->ofile << "\"R" << r+1 << "\"";
+         else
+            this->ofile << "R" << r+1;
+         this->ofile << csep;   // Blank empty field at the beginning of each line
+        }
+
         for (indextype c=0;c<this->nc-1;c++)
-            this->ofile << Get(r,c) << csep;
-        this->ofile << Get(r,this->nc-1) << std::endl;
+            this->ofile << std::setprecision(p) << Get(r,c) << csep;
+        this->ofile << std::setprecision(p) << Get(r,this->nc-1) << std::endl;
     }
     this->ofile.close();
 }
